/*
    SPDX-FileCopyrightText: 2017 Volker Krause <vkrause@kde.org>

    SPDX-License-Identifier: MIT
*/

#ifndef KUSERFEEDBACK_CPUINFOSOURCE_H
#define KUSERFEEDBACK_CPUINFOSOURCE_H

#include "kuserfeedbackcore_export.h"
#include "abstractdatasource.h"

namespace KUserFeedback {

/*!
 * \class KUserFeedback::CpuInfoSource
 * \inmodule KUserFeedbackCore
 * \inheaderfile KUserFeedback/CpuInfoSource
 *
 * \brief Data source reporting the type and amount of CPUs.
 *
 * The default telemetry mode for this source is Provider::DetailedSystemInformation.
 */
class KUSERFEEDBACKCORE_EXPORT CpuInfoSource : public AbstractDataSource
{
    Q_DECLARE_TR_FUNCTIONS(KUserFeedback::CpuInfoSource)
public:
    /*!
     *
     */
    CpuInfoSource();

    QString name() const override;
    QString description() const override;

    QVariant data() override;
};

}

#endif // KUSERFEEDBACK_CPUINFOSOURCE_H
